      SUBROUTINE ANDERSN(X0,X1,ITC)
c     SUBROUTINE ANDERSN(SM,FM,ITC)
c*********************************************************************
c
c     g e n e r a l i z e d   a n d e r s o n   m e t h o d
c
c     implemented here according to notes of s.b.
c     broyden's iteration scheme following the papers of :
c     srivastava, j. phys. , 17 (1984) , pp l317
c     c.g. broyden in math.comput., 19 , pp 577, 1965
c     c.g. broyden in ibid, 21 ,pp 368 ,1967
c     the method has been generalized to include a metric.the
c     definition of the necessary inner products are similar to the
c     discription given in the notes of m.weinert.the algorithm
c     discribed in the paper srivastava  has been simplified
c     ( see notes of s.b.)
c     the anderson method (d.g. anderson, j. acm 12, 547 (1964)) has
c     been generalized and reformulated as an improvement of broyden's
c     second method. successive linesearch is replaced by successive
c     search on hyperplanes. ( see notes of s.b. )
c                  S. Bluegel , ISSP , july 1989
c*********************************************************************
C     .. Parameters ..
      INCLUDE 'inc.p'
      INCLUDE 'inc.cls'
      INTEGER LMMAXD
      PARAMETER (LMMAXD= (LMAXD+1)**2)
      INTEGER NDIM
      PARAMETER (NDIM=LMMAXD*NAEZD)
      INTEGER NTIRD,N2
      PARAMETER (N2=NDIM)
      PARAMETER (NTIRD=NDIM*2)
c     INTEGER ITDTHD
c     PARAMETER (ITDTHD=442)
C     ..
C     .. Scalar Arguments ..
      INTEGER IMAP,ITC
C     ..
C     .. Local Scalars ..
      DOUBLE PRECISION CMM,ONE,VMDENO
      INTEGER IJ,IT
C     ..
C     .. External Functions ..
      DOUBLE PRECISION DDOT
      EXTERNAL DDOT
C     ..
C     .. External Subroutines ..
      EXTERNAL DAXPY,DSCAL
C     ..
C     .. Intrinsic Functions ..
      INTRINSIC ABS
C     ..
C     .. Save statement ..
      SAVE
C     ..
C     .. Local Arrays ..
      DOUBLE COMPLEX X0(NDIM),X1(NDIM)
      DOUBLE PRECISION FM(NTIRD),FM1(NTIRD),SM(NTIRD),SM1(NTIRD)
      DOUBLE PRECISION AM(2:ITDTHD-1),WIT(2:ITDTHD),
     +                 UI2(NTIRD,ITDTHD),VI2(NTIRD,ITDTHD)
      DOUBLE PRECISION AM1(ITDTHD-1),WIT1(ITDTHD-1)
C     ..
          IJ = 0
          DO I = 1,N2
            IJ = IJ + 1
            SM(IJ) = DREAL(X0(I))
            FM(IJ) = DREAL(X1(I))
          END DO
          DO I = 1,N2
            IJ = IJ + 1
            SM(IJ) = DIMAG(X0(I))
            FM(IJ) = DIMAG(X1(I))
          END DO
          IMAP = IJ
              DO IJ = 1,IMAP
                FM(IJ) = FM(IJ) - SM(IJ)
              END DO
      IF (ITC.EQ.1) THEN
                CALL DCOPY(IMAP,SM,1,SM1,1)
                CALL DCOPY(IMAP,FM,1,FM1,1)
                DO IJ = 1,IMAP
                  SM(IJ) = SM(IJ) + FM(IJ)
                END DO
      ELSE

c
c----> calculate  sm = rho(m) - rho(m-1)
c----> calculate dfm = f[m] - f[m-1]
c
        DO 10 IJ = 1,IMAP
          SM1(IJ) = SM(IJ) - SM1(IJ)
          FM1(IJ) = FM(IJ) - FM1(IJ)
   10   CONTINUE
c
c----> generate u[m] = u(ij,mit) and v[m]
c
        DO 20 IJ = 1,IMAP
          UI2(IJ,ITC) = FM1(IJ) + SM1(IJ)
          VI2(IJ,ITC) = FM1(IJ)
   20   CONTINUE
          AM1(1) = 0.0D0
          WIT1(1) = 0.0D0
        DO 30 IT = 2,ITC - 1
          AM(IT) = DDOT(IMAP,FM1,1,VI2(1,IT),1)
          AM1(IT) = -AM(IT)
          WIT1(IT) = -AM(IT)*WIT(IT)
c         CALL DAXPY(IMAP,-AM(IT),UI2(1,IT),1,UI2(1,ITC),1)
c         CALL DAXPY(IMAP,-AM(IT)*WIT(IT),VI2(1,IT),1,VI2(1,ITC),1)
   30   CONTINUE
        CALL DGEMV('N',IMAP,ITC-1,1.D0,UI2,NTIRD,AM1,1,
     +                    1.0D0,UI2(1,ITC),1)
        CALL DGEMV('N',IMAP,ITC-1,1.D0,VI2,NTIRD,WIT1,1,
     +                    1.0D0,VI2(1,ITC),1)
c
c----> print amj = the importance of the history of ui
c
c       WRITE (6,FMT='(5x,'' amj , ---> j=2,'',i3,/,(9x,1p,7d10.2))')
c    +    ITC - 1, (AM(IT),IT=2,ITC-1)
c
c----> complete the evaluation of v[m]
c
          VMDENO = DDOT(IMAP,FM1,1,VI2(1,ITC),1)

          IF (ABS(VMDENO).LT.1D-70) STOP 'ANDRSN'

          CALL DSCAL(IMAP,1.D0/VMDENO,VI2(1,ITC),1)
c
c----> save wit(mit) for next iteration
c
          WIT(ITC) = VMDENO
c
c----> update f[m-1] = f[m]  ; rho(m) = rho(m-1)
c
        DO 60 IJ = 1,IMAP
          FM1(IJ) = FM(IJ)
          SM1(IJ) = SM(IJ)
   60   CONTINUE
c
c----> calculate cmm
c
        CMM = DDOT(IMAP,FM,1,VI2(1,ITC),1)
C           WRITE (6,FMT='(5X,'' CMM = '',1P,D12.4)') CMM
c
c----> update rho(m+1)
c
        CALL DAXPY(IMAP,1.D0-CMM,UI2(1,ITC),1,SM,1)

      END IF

          IJ = 0
          DO I = 1,N2
            IJ = IJ + 1
            X0(I) = DCMPLX(SM(IJ),SM(IJ+IMAP/2))
          END DO

        RETURN

      END

