      SUBROUTINE ANALYZEVERT3D(
     >                          NVERTMAX,NFACED,TOLVDIST,TOLAREA,NPLANE,
     X                          NFACE,NVERT,XVERT,YVERT,ZVERT,
     X                          A3,B3,C3,D3)
c Analyze the faces and vertices of the polyhedron.
c Use criteria for rejecting faces that are too small 
c or vertices that are too close to each other.
c On output, number of faces and vertices may be reduced 
c after some rejections have taken place.
      implicit none
c Input:
      INTEGER NVERTMAX,NFACED
      INTEGER NPLANE                 ! Number of planes
c Input and output:
      INTEGER NFACE                  ! Number of faces (usually much less than nplane)
      INTEGER NVERT(NFACED)          ! Number of vertices for each face
      REAL*8  XVERT(NVERTMAX,NFACED),YVERT(NVERTMAX,NFACED)
     &       ,ZVERT(NVERTMAX,NFACED)
      REAL*8 TOLVDIST,TOLAREA  ! Max. tolerance for distance of two vertices and area of face.
      REAL*8 A3(*),B3(*),C3(*),D3(*)  ! Coefs. defining the planes, to be reordered at end
c Inside
      REAL*8 VDIST             ! Distance between consecutive vertices

      LOGICAL LACCEPTVERT(NVERTMAX),LACCTOT,LFOUNDNEXT
      LOGICAL LACCEPTFACE(NFACED)
      INTEGER NEWINDEXFACE(NFACED)

      INTEGER IFACE,IVERT,IVERT2,INEXT,IFACENEWCOUNT,IVERTNEWCOUNT,IPLANE
      REAL*8 DX,DY,DZ,X1,X2,X3,Y1,Y2,Y3,Z1,Z2,Z3,TRIANGLEAREA
      REAL*8 FACEAREA(NFACED)


      REAL*8 X4,Y4,Z4,DET,DETSUM


c First analyze vertices.
c Reject doubles (also vertices which fall almost on the previous vertex).
      DO 100 IFACE = 1,NFACE

         LACCTOT = .TRUE.
c First vertices 1st with 2nd, 2nd with 3rd, etc...
         LACCEPTVERT(1) = .TRUE.                 ! First vertex is always accepted.
         IVERT = 1

c Double loop: First loop is over all vertices; but during the loop, rejected vertices are jumped over.
         DO WHILE (IVERT.LT.NVERT(IFACE))
            LFOUNDNEXT = .FALSE.                 ! This flag will become true when the next vertex is found.
            IVERT2 = IVERT + 1
c Second loop is over subsequent vertices.
c Look for the first acceptable-as-next vertex, but do not go beyond last vertex.
c Stop loop as soon as the acceptable next vertex is found.
            DO WHILE (.NOT.LFOUNDNEXT.AND.IVERT2.LE.NVERT(IFACE))
               DX = XVERT(IVERT2,IFACE) -  XVERT(IVERT,IFACE)
               DY = YVERT(IVERT2,IFACE) -  YVERT(IVERT,IFACE)
               DZ = ZVERT(IVERT2,IFACE) -  ZVERT(IVERT,IFACE)
               VDIST = DSQRT(DX*DX + DY*DY + DZ*DZ)

               IF (VDIST.GE.TOLVDIST) THEN
                  LACCEPTVERT(IVERT2) = .TRUE.   ! Vertex is to be accepted
                  INEXT = IVERT2                 ! Set this as the next vertex
                  LFOUNDNEXT = .TRUE.            ! and we have a winner, exit loop.
               ELSE
                  LACCEPTVERT(IVERT2) = .FALSE.  ! Remember that vertex is to be rejected later
                  LACCTOT = .FALSE.              ! Remember that at least one vertex has been rejected.
                  IVERT2 = IVERT2 + 1             ! Now compare to the next vertex
              ENDIF               
            ENDDO

            IVERT = INEXT
            
         ENDDO

c ...and now 1st with last to close the cycle:
         IVERT = 1
         IVERT2 = NVERT(IFACE)
         DX = XVERT(IVERT2,IFACE) -  XVERT(IVERT,IFACE)
         DY = YVERT(IVERT2,IFACE) -  YVERT(IVERT,IFACE)
         DZ = ZVERT(IVERT2,IFACE) -  ZVERT(IVERT,IFACE)
         VDIST = DSQRT(DX*DX + DY*DY + DZ*DZ)
         IF (VDIST.GE.TOLVDIST) THEN
            LACCEPTVERT(IVERT2) = .TRUE.        ! Vertex is to be accepted
         ELSE
            LACCEPTVERT(IVERT2) = .FALSE.       ! Remember that vertex is to be rejected later
            LACCTOT = .FALSE.                   ! Remember that at least one vertex has been rejected.
         ENDIF


c Reject inappropriate vertices and re-index
         IF (.NOT.LACCTOT) THEN
            IVERTNEWCOUNT = 0
            DO IVERT = 1,NVERT(IFACE)
               IF (LACCEPTVERT(IVERT)) THEN
                  IVERTNEWCOUNT = IVERTNEWCOUNT + 1    ! One more vertex to accept 
                  IF (IVERTNEWCOUNT.NE.IVERT) THEN     ! Otherwise the correct value is already at the correct place
                     XVERT(IVERTNEWCOUNT,IFACE) = XVERT(IVERT,IFACE) ! Re-index vertex
                     YVERT(IVERTNEWCOUNT,IFACE) = YVERT(IVERT,IFACE)
                     ZVERT(IVERTNEWCOUNT,IFACE) = ZVERT(IVERT,IFACE)
                  ENDIF
               ENDIF
            ENDDO
            NVERT(IFACE) = IVERTNEWCOUNT
         ENDIF

 100  ENDDO



c Now analyze faces, reject faces with less than three vertices and faces of very small area.
      DO 200 IFACE = 1,NFACE
         IF (NVERT(IFACE).GE.3) THEN  ! calculate area
            X1 = XVERT(1,IFACE)
            Y1 = YVERT(1,IFACE)
            Z1 = ZVERT(1,IFACE)
            FACEAREA(IFACE) = 0.d0
            DO IVERT = 2,NVERT(IFACE)-1
               X2 = XVERT(IVERT,IFACE)
               Y2 = YVERT(IVERT,IFACE)
               Z2 = ZVERT(IVERT,IFACE)
               X3 = XVERT(IVERT+1,IFACE)
               Y3 = YVERT(IVERT+1,IFACE)
               Z3 = ZVERT(IVERT+1,IFACE)
               TRIANGLEAREA = 0.5d0 * DABS(
     &              (X2-X1)*(X3-X1)+(Y2-Y1)*(Y3-Y1)+(Z2-Z1)*(Z3-Z1) )
               FACEAREA(IFACE) = FACEAREA(IFACE)+ TRIANGLEAREA
            ENDDO

            WRITE(*,8000) IFACE,FACEAREA(IFACE)

            IF (FACEAREA(IFACE).GE.TOLAREA) THEN
               LACCEPTFACE(IFACE) = .TRUE.
            ELSE
               LACCEPTFACE(IFACE) = .FALSE.  ! Reject facees with small area
               WRITE(*,8010) TOLAREA 
            ENDIF
            
         ELSE
            LACCEPTFACE(IFACE) = .FALSE.  ! Reject facees with less than 3 vertices
            WRITE(*,8020) IFACE,NVERT(IFACE)
         ENDIF

 200  ENDDO


c Re-order the faces so that the accepted ones are in the first positions
      IFACENEWCOUNT = 0
      DO IFACE = 1,NFACE
         IF (LACCEPTFACE(IFACE)) THEN
            IFACENEWCOUNT = IFACENEWCOUNT + 1  ! One more face to accept
            IF (IFACENEWCOUNT.NE.IFACE) THEN   ! Otherwise the correct value is already at the correct place
               NVERT(IFACENEWCOUNT) = NVERT(IFACE) ! Re-index face vertex number
               DO IVERT = 1,NVERT(IFACE)
                  XVERT(IVERT,IFACENEWCOUNT) = XVERT(IVERT,IFACE) ! Re-index face vertices
                  YVERT(IVERT,IFACENEWCOUNT) = YVERT(IVERT,IFACE) ! Re-index face vertices
                  ZVERT(IVERT,IFACENEWCOUNT) = ZVERT(IVERT,IFACE) ! Re-index face vertices
               ENDDO
               A3(IFACENEWCOUNT) = A3(IFACE) ! Re-index face equation parameters
               B3(IFACENEWCOUNT) = B3(IFACE) ! Re-index face equation parameters
               C3(IFACENEWCOUNT) = C3(IFACE) ! Re-index face equation parameters
               D3(IFACENEWCOUNT) = D3(IFACE) ! Re-index face equation parameters
            ENDIF
         ENDIF
      ENDDO
      NFACE = IFACENEWCOUNT
 
c Check for every face that all veritces lie on the same plane
c by checking linear dependence
      DO IFACE = 1,NFACE
         X2 = XVERT(2,IFACE) - XVERT(1,IFACE)
         Y2 = XVERT(2,IFACE) - YVERT(1,IFACE)
         Z2 = XVERT(2,IFACE) - ZVERT(1,IFACE)
         X3 = XVERT(3,IFACE) - XVERT(1,IFACE)
         Y3 = XVERT(3,IFACE) - YVERT(1,IFACE)
         Z3 = XVERT(3,IFACE) - ZVERT(1,IFACE)
         DETSUM = 0.d0
         DO IVERT = 4,NVERT(IFACE)
            X4 = XVERT(IVERT,IFACE) - XVERT(1,IFACE)
            Y4 = XVERT(IVERT,IFACE) - YVERT(1,IFACE)
            Z4 = XVERT(IVERT,IFACE) - ZVERT(1,IFACE)
            DET = X2*(Y3*Z4-Y4*Z3)+Y2*(Z3*X4-Z4*X3)+Z2*(X3*Y4-X4*Y3)
            DETSUM = DETSUM + DABS(DET)
            IF (DABS(DET).GT.1.D-16) WRITE(*,9000) IFACE,IVERT,DET
         ENDDO
         WRITE(*,9010) IFACE,DETSUM
      ENDDO
      

 8000 FORMAT('ANALYZEVERT3D: Face',I5,' has area',E12.4)
 8010 FORMAT('Face will be rejected ; Max. area tolerance=',E12.4)
 8020 FORMAT('Face',I5,' has only',I3,' vertices and is rejected')
 9000 FORMAT('Error from ANALYZEVERT3D: Vertices not on single plane.',
     &        ' IFACE=',I5,' IVERT=',I5,' DETERMINANT=',E12.4)
 9010 FORMAT('ANALYZEVERT3D: Checking that vertices lie on plane.',
     &     ' IFACE=',I5,' ; Determinants sum to be zero=',E12.4)

      END

