c ************************************************************************
      SUBROUTINE CLSGEN2000(NAEZ,RR,NR,RBASIS,
     &                      Z,CLS,NCLS,
     &                      NACLS,ATOM,
     &                      RCLS,RCUT,RCUTXY,
     &                      ALAT)
C
      IMPLICIT NONE
c ************************************************************************
c This subroutine is used to create the clusters around each atom 
c where in order to prepare the shape functions and produce povray 
c input files. (Based on clsgen99.f)
c
c STRATEGY : 
c Calculate the cluster of each atom by the lattice
c parameters avaliable. Sort the atoms in a unique way :big r, big z, big y
c compare the positions with the previous clusters to see if there is 
c a difference. If not keep only previous clusters and make indexing if
c a new cluster is found then check dimensions and continue for the new
c atom.  
c
c
      include 'inc.geometry'    
c
c
c     .. arguments
c
      REAL*8       ALAT         ! lattice constant A
      REAL*8       RCUT,RCUTXY
      REAL*8      
     +     RBASIS(3,*),             ! pos. of basis atoms in EZ
     +     RCLS(3,NACLSD,*),        ! real space position of atom in cluster
     +     RR(3,0:NRD),             ! set of lattice vectors
     +     Z(*)                     ! nucleus charge
c
      INTEGER
     +     NAEZ,                    ! number of atoms in EZ
     +     NCLS,                    ! number of diff. clusters
     +     NR                       ! number of lattice vectors RR
c
      INTEGER
     +     CLS(*),                  ! sort of cluster around atom
     +     NACLS(*),                ! number of atoms in cluster
     +     ATOM (NACLSD,*),         ! index to atom in elem/cell at site in cluster
     +     EZOA (NACLSD,NAEZD)      ! index to bravais lattice  at site in cluster
c
c     .. locals
c
      INTEGER 
     +     I,J,N1,
     +     NA,NUMBER,N,NC,NPRIN,ITEST1,ITEST,
     +     POS,IA,IN,IB,II,JATOM,ICU,IC,IAT,I0,I1,ICLUSTER
      INTEGER IATOM(NACLSD),IEZOA(NACLSD),
     +     ISORT(NACLSD),ICOUPLMAT(NAEZD,NAEZD)
c
      REAL*8        
     +     R,R1,R2,RABS,RD,T,EPSSHL,
     +     ASC(3),RCLS1(3,NACLSD),
     +     R0(3,20),RG(3,NACLSD),TMP(3),RSORT(NACLSD),plotx,ploty,plotz
      INTEGER NLAY,                                
     +        NLBASIS,NRBASIS,                    
     +        NLEFT,NRIGHT           
      REAL*8       RCUT2,RCUTXY2,RXY2 
c
      LOGICAL  CLUSTCOMP
      character*3 elem_name
      character*24 pov_color
      REAL*8       pov_radious
      real*8 color(3),radious
      integer ntot
c
c
      LOGICAL TEST,OPT,pov
      EXTERNAL DSORT,CLUSTCOMP
      INTRINSIC MIN,SQRT
c
      DATA     EPSSHL   / 1.0D-4 /
      data pov_radious/0.3d0/
      DATA pov_color/'Blue                    '/
c                    123456789012345678901234
c ------------------------------------------------------------------------
      write(6,*) '>>> CLSGEN99: generation of cluster coordinates'
c This is generating the clusters which have a distance smaller
c than RCUT and RCUTXY in plane .
c The cluster atoms are ordered with radious and then z>y>x 
c The ordering allows an easy comparison of clusters
c The principal layer for each layer (atom in unit cell) is
c calculated also for each cluster and the maximum number
c is returned. Some dimension tests are also done      

      WRITE(6,*) 'RCUT = ',rcut,' RCUTXY = ',rcutxy

      ICLUSTER = 1
      RCUTXY2 = (RCUTXY+EPSSHL)*(RCUTXY+EPSSHL)
      RCUT2   = (RCUT+EPSSHL)*(RCUT+EPSSHL)


      DO 2 JATOM = 1,NAEZ       ! loop in all atoms or layers
c Printing the layers
c
c
         CLS(JATOM) = 0   
c     ----------------------------------------------         
         NUMBER = 0             ! counter for atoms in cluster
         DO NA = 1,NAEZ  ! loop in all atoms
            DO N=0,NR    ! loop in all bravais vectors    
               DO I=1,3
                  TMP(I) = RR(I,N)+RBASIS(I,NA)-RBASIS(I,JATOM)
               END DO
               RXY2 =  TMP(1)**2+TMP(2)**2
               R2   =  TMP(3)**2 + TMP(1)**2+TMP(2)**2

               IF ( (RXY2.LE.RCUTXY2).AND.(R2.LE.RCUT2) )  THEN
                  NUMBER = NUMBER + 1
                  ATOM(NUMBER,JATOM) = NA ! store the atom in elem cell
                  EZOA(NUMBER,JATOM) = N ! store the bravais vector
                  DO I=1,3
                     RCLS1(I,NUMBER) = TMP(I)
                  END DO
               END IF
 800           format(3I5,4F8.4)
            END DO              ! N loop in bravais
            
         END DO                 ! NA loop in NAEZ
  
c     
c     In the case of 2 dimensional case loop in the atoms
c     outside.
c     
c     Now the atom JATOM Has it's cluster first 
c     sort the atoms of the cluster in increasing order. First by distance
c     Then by z then by y    
c     
         IF (NUMBER.GT.NACLSD) THEN 
            write(6,*) 'Please, increase the parameter NACLSD in',
     &              ' inc.cls to a value greater equal ',number,' .'
               STOP 'Dimension error.'
        
         END IF
         do ia=1,number
           rsort(ia) = SQRT(RCLS1(1,ia)**2+
     &                       RCLS1(2,ia)**2+
     &                       RCLS1(3,ia)**2)
c           rsort(ia) = 1000.d0*rsort(ia)-
c    &                    10.d0*RCLS1(3,ia)-
c    &                   0.1d0*RCLS1(2,ia)-
c    &                  0.001d0*RCLS1(1,ia) 
c changed on 28.11.2000
           rsort(ia) = 100000000.d0*rsort(ia)+
     &                    100000.d0*RCLS1(3,ia)+
     &                       100.d0*RCLS1(2,ia)+
     &                         0.1d0*RCLS1(1,ia)
         end do      
c
c         do IA=1,NUMBER
c           WRITE(6,*) JATOM,'YY',RCLS1(1,IA),RCLS1(2,IA),RCLS1(3,IA)
c           WRITE(6,*) JATOM,'XXXXXX',RSORT(IA)
c         end do
c
         CALL DSORT(RSORT,ISORT,NUMBER,POS)
c     Rearange exchange ia with ib
c     MAP temporarily to another array         
         do IA=1,NUMBER       
            do I=1,3
               RG(I,IA)    = RCLS1(I,IA)
            end do
            IATOM(IA) = ATOM(IA,JATOM)
            IEZOA(IA) = EZOA(IA,JATOM)
         end do    
c Now use correct order
         do IA =1,NUMBER
c           WRITE(6,*) JATOM,'ZZ',ISORT(IA)
            IB = ISORT(IA)
             do I=1,3
               RCLS1(I,IA) = RG(I,IB)
            end do
            ATOM(IA,JATOM) = IATOM(IB)
            EZOA(IA,JATOM) = IEZOA(IB) 
         END DO
c     
c     Now the clusters have a unique sorting and can be compared with
c     each other Check if ICLUSTER was found previously       
c     
         DO ICU = 1,ICLUSTER-1
            
            N1 = NACLS(ICU)
            IF(CLUSTCOMP(RCLS,ICU,N1,RCLS1,NUMBER) ) THEN 
c                                                       ! return true if found before
               CLS(JATOM) = ICU
            END IF
         END DO
         IF (CLS(JATOM).EQ.0) THEN
            NCLS = ICLUSTER
            IF (ICLUSTER.GT.NCLSD) THEN
               write(6,*) 'Please, increase the parameter NCLSD in',
     &              ' inc.cls to a value greater equal ',ICLUSTER,' .'
               STOP 'Dimension error.' 
            END IF
            CLS(JATOM) = ICLUSTER
            NACLS(ICLUSTER) = NUMBER
            DO IN = 1,NUMBER
               DO II=1,3
               RCLS(II,IN,ICLUSTER) = RCLS1(II,IN)
               END DO
               write(6,800) jatom,atom(in,jatom),ezoa(in,jatom),
     &                  (rcls1(i,in),i=1,3),
     &      sqrt(rcls1(1,in)**2+rcls1(2,in)**2+rcls1(3,in)**2)
            END DO   
            ICLUSTER = ICLUSTER + 1
         END IF 
c ******************************************************
       write(6,*) 'Atom ',JATOM,' has cluster ', CLS(JATOM),
     &            'with ',NUMBER,' sites'
 2    CONTINUE                      ! JATOM = 1,NAEZ

c
c Now all clusters of all atoms are found print out
c and test the results...
c
c
       WRITE(6,*) ' Sub clsgen99  exiting <<<<<<<<<<<<<'
c ------------------------------------------------------------------------

      RETURN
      END


