      PROGRAM VORONANO
C
      implicit none
C
      include 'inc.geometry'
C
      INTEGER   NATMAX,NFACED,NVERTD
      PARAMETER (NATMAX=500,NVERTD=250,NFACED=300)
      INTEGER   LMXSPD,IBMAXD
      PARAMETER (LMXSPD= (2*LPOTD+1)**2,IBMAXD=(LMAXD1+1)*(LMAXD1+1))
      INTEGER   NTOTD
      PARAMETER (NTOTD=NATYPD+500)
      DOUBLE COMPLEX CONE,CONEM,CZERO,CI
      PARAMETER (CONE  = ( 1.0D0,0.0D0),
     +           CONEM = (-1.0D0,0.0D0),
     +           CZERO = ( 0.0D0,0.0D0),
     +           CI    = ( 0.0D0,1.0D0))
c *****************************************************************
c * Program description and small help.
c * This is a utility of the tb-kkr and impurity programs. The 
c * purpose to construct the potential files possibly shape
c * functions and visualize the lattice using an external ray-tracer
c * this version is using povray.
c * As input the tb-kkr "inputcard" is used with some extra parameters
c * In case of impurity calculations an extra file with the atomic   
c * positions is neaded.
c * I describe some of the problems the program handles: 
c *
c * If you have no potentials just set the parameters you want
c * and leave blanc space as potential file 
c *
c *
c * 1. ASA potentials for some lattice, start from potentials 
c * in the GENERAL MESH or start from scratch
C *
C * i.  Define the lattice in the inputcard
C * ii. Put old potential as input and use ASA parameters in the
C *     inputcard
C * The program calculates asa spheres, etc and produces the potential
C * in the correct format to start the tb-kkr
C *
C * 2. I nead FP for some lattice 
C *
C * i Define lattice and give parameters for full potential
C * ii. Use some old potential or start from scratch
C * iii. Define the "muffin-tin-ization" in case of shifting
c *      the atoms later. This must be given in a.u. for each atom
C * The program calculates the first neighbours for each atom, makes
C * a Voronoi construction (posible weights) and then constructs the
C * shape functions. It sets the shape functions to the given mt-radious
c * and continues by constructing the potentials in the obtrained 
c * radial mesh.
c *
c * 3. Potentials for impurity calculation
c *  i. prepare "inputcard" for all parameters, and use the option
c *     "IMPURITY" make a file with the impurity atomic positions
c *     filename:  "impurity.atoms"
c *     in the format given below
c * index           R_new                             R_old           shell  weight
c *        x      y          z           x          y           z
c * ------------------------------------------------------------------------------  
c * 1  0.0000  0.04000  0.00000000  0.00000000  0.00000000  0.00000000    1   1.0
c   2 -0.5000 -0.55000 -0.50000000 -0.50000000 -0.50000000 -0.50000000    2   1.0
c   3 -0.5000 -0.50000  0.50000000 -0.50000000 -0.50000000  0.50000000    2   1.0
c   4  0.5000 -0.50000 -0.50000000  0.50000000 -0.50000000 -0.50000000    2   1.0
c   5 -0.5000  0.50000 -0.50000000 -0.50000000  0.50000000 -0.50000000    2   1.0
c   6  0.5000 -0.50000  0.50000000  0.50000000 -0.50000000  0.50000000    2   1.0
c   7 -0.5000  0.50000  0.50000000 -0.50000000  0.50000000  0.50000000    2   1.0
c   8  0.5000  0.53000 -0.50000000  0.50000000  0.50000000 -0.50000000    2   1.0
c   9  0.5000  0.50000  0.50000000  0.50000000  0.50000000  0.50000000    2   1.0
c    . 
c    . etc.
c    .
c  28 -0.5000 -1.50000000 -0.50000000 -0.50000000 -1.50000000 -0.50000    0   1.0
c  29 -1.5000 -0.50000000 -0.50000000 -1.50000000 -0.50000000 -0.5000    0    1.0
c  30 -0.5000 -1.50000000  0.50000000 -0.50000000 -1.50000000  0.50000    0   1.0
c  31 -1.500 -0.50000000  0.50000000 -1.50000000 -0.50000000  0.50000    0    1.0
c  32 -0.5000 -0.50000000 -1.50000000 -0.50000000 -0.50000000 -1.5000    0    1.0
c  33  0.5000 -1.50000000 -0.50000000  0.50000000 -1.50000000 -0.5000    0    1.0
c
c     etc.
c
c * since the cluster has to be surrounded by some atoms use atomic positions
c * with shell type 0 for surrounding atoms.
c * the program will do the rest of the job and will give you shape functions
c * and potentials (using GENERAL FORM pots or jellium pots as start) 
c * R_new is the position of the atoms and R_old is the lattice used
c * for the voronoi construction i.e. extra shifts are taken care of.
c *
c * The program creates two files : lattice.pov and voronoi.pov
c * you nead to put the file "povray.ini" in your path and run
c * povray with :     povray +I lattice.pov
c * 
c * Rasmol with connectivity info also avaliable 3.3.2002, protein 
c * database format 
c * visualize file: lattice.pdb 
c *
c * the camera positions etc are in the end of the files.
c * help on povray : http://www.povray.org  
c *                                                    ver.  10.2000
c *
c *  parameters due to change to do different things:
c *    npoi    : data number of points for the shape function
c *              usualy set to 125 and can be changed 
c *    dlt = 0.05 controls the accuracy of angular integration
c *            for producing the shape functions can be also changed
c *    nrad : number of points for the muffin-tinizing 
c *           used in sub mtmesh
c *                                                  planed : smeared shapes
c *
c * KNOWN PROBLEM : Sometimes the voronoi construction fails.
c * This usualy means that the coordinates are not accurate enought
c * Try to change the weights by 0.001 or so and run again 
c *           
c * 
c *******************************************************************     
      REAL*8    
     +     ALATC,                   ! lattice constants (in a.u.)
     +     C,                       ! speed of light
     +     ABASIS,BBASIS,CBASIS,    ! scaling factors for rbasis
     +     FAC,FACL,                ! factor (+/-1.0d0)
     +     GAMMA,                   ! lattice distortion
     +     LATSP,                   ! lattice distortion
     +     TOLVDIST,                ! Max. tolerance for distance of two vertices
     +     TOLAREA,                 ! Max. tolerance for area of polygon face
     +     TOLEULER                 ! Used in calculation of Euler angles, subr. EULER
      REAL*8    
     +     VOLUC,                   ! volume of unit cell in units of alat**3
     +     FPI,PI,RFPI,
     +     RFOURIER                 ! cutting radius for Fourier-transf.
C
C     .. REAL*8    ARRAYS ....
C
      REAL*8   
     +     ATWGHT(NATYPD),
     +     BRAVAIS(3,3),            ! bravais lattice vectors
     +     MTFAC(NATYPD),           ! scaling factor for radius MT
c
     +     A(NATYPD),B(NATYPD),     ! contants for exponential r mesh
     +     R(IRMD,NATYPD),          ! radial r mesh (in units a Bohr)
     +     RBASIS(3,NAEZD),         ! position of atoms in the unit cell
                                    ! in units of bravais vectors
     +     RBASIS1(3,NAEZD),        ! pos. of atoms in atomic units
     +     RCLS(3,NACLSD,NCLSD),    ! real space position of atom in cluster
     +     RCLS1(3,NACLSD),         ! only for test use
     +     RMT(NATYPD),             ! Muffin-Tin-radius
     +     RMTNEW(NATYPD),          ! adapted MT radius
     +     RR(3,0:NRD),             ! set of real space vectors (in a.u.)
     +     RS(IRMD,0:LMAXD,NATYPD), ! r mesh for relat. calc.
     +     RWS(NATYPD),             ! Wigner Seitz radius
     +     S(0:LMAXD,NATYPD),
     +     Z(NATYPD)
      INTEGER  ICLS, NAEZ,              ! number of atoms in unit cell
     +         NATYP,                   ! number of kinds of atoms in unit cell
     +         NCLS                     ! number of reference clusters
      INTEGER
     +     KAOEZ(NAEZD),            ! kind of atom at site in elem. cell
     &     NTCELLR(NREFD),
c
c     ..  cluster arrays
c
     +     ATOM(NACLSD,NAEZD),     ! atom at site in cluster
     +     CLS(NATYPD),             ! cluster around atom
     +     COCLS(NCLSD),            ! center of cluster (kaez )
     +     NACLS(NCLSD),            ! number of atoms in cluster
c
     +     IFUNM(NATYPD,LMXSPD),
     +     IMT(NATYPD),             ! r point at MT radius
     +     IPAN(NATYPD),            ! number of panels in non-MT-region
     +     IRC(NATYPD),             ! r point for potential cutting
     +     IRCUT(0:IPAND,NATYPD),   ! r points of panel borders
     +     IRMIN(NATYPD),           ! max r for spherical treatment
     +     IRNS(NATYPD),            ! number r points for non spher. treatm.
     +     IRWS(NATYPD),             ! r point at WS radius
     &     KFG(4,NATYPD),NTCELL(NATYPD)
c     
      INTEGER  IER,NREPATOM,REPATOM(NATYPD),N,II1,II2
      REAL*8   RMTCORE(NATYPD)
c
      INTEGER  NVEC
      REAL*8   RVEC(3,NFACED)
c
c  Shape functions
c
      INTEGER  NCELL_S,NPAN_S,NFUN_S 
      INTEGER  NM_S(NPAND),
     +         MESHN_S(NATYPD),
     +         LMIFUN_S(IBMAXD)
      REAL*8   SCALE_S,SCALEALL(NAEZD)
      REAL*8   XRN_S(IRID,NATYPD),
     +         DRN_S(IRID,NATYPD),
     +         THETAS_S(IRID,IBMAXD)
c Output:
      INTEGER  NFACE
      INTEGER  NEDGE(NFACED),
     +         NFACECL(NCLSD),
     +         NEDGECL(NFACED,NCLSD)
      REAL*8   VOLUME
      REAL*8   A3(NFACED),B3(NFACED),C3(NFACED),D3(NFACED)
      REAL*8   A3CL(NFACED,NAEZD),B3CL(NFACED,NAEZD),
     &         C3CL(NFACED,NAEZD),D3CL(NFACED,NAEZD)
      REAL*8   XEDGE(NVERTD,NFACED),YEDGE(NVERTD,NFACED),
     &         ZEDGE(NVERTD,NFACED)
      REAL*8   XEDGECL(NVERTD,NFACED,NAEZD),
     &         YEDGECL(NVERTD,NFACED,NAEZD),
     &         ZEDGECL(NVERTD,NFACED,NAEZD),
     &         VOLUMECL(NATYPD),RWSCL(NATYPD),RMTCL(NATYPD)
      REAL*8   DX(NTOTD),DY(NTOTD),DZ(NTOTD)
      REAL*8   ROUT,RTEST,DLT,CRAD,RX,RY,RZ,MTRADIUS,VTOT,
     &         SHAPESHIFT(3,NATYPD)
      CHARACTER*80 UIO
      INTEGER  LMAX,KEYPAN,NPOI,NA,IAT,ICL,NSHAPE,II,
     +         I,J,IFILE,III,INS,KWS,
     +         LPOT,NSPIN,
     +         NR,KMT,NINEQ,I1,IC0,
     +         IRM,KXC,ICLUSTER,LMAXSHAPE,
     +         NRAD,NMIN
      INTEGER  NM(NPAND),IDSHAPE(NATYPD),SHAPECL(NATYPD),
     +         LMXC(NAEZD),REFPOT(NAEZD)
      LOGICAL  LSHIFT(NATYPD),MAKESHAPE,VORO
      REAL*8   RCUTZ,RCUTXY,QBOUND,SHIFT(3),DIFF,RMT0
      REAL*8   WEIGHT0,WEIGHT(NATMAX)
      REAL*8   SIZEFAC(0:NAEZD),SHAPEWEIGHT(NACLSD,NATYPD)
      REAL*8   ND(3,3),R1,R2,R3
      CHARACTER*24 TXC(3)
      LOGICAL OPT
      CHARACTER*40 I13
C
      EXTERNAL SHAPE
      SAVE
      DATA NM/NPAND*25/
c
c  Data paremeters due to change for npoi check dimensions in inc.geometry
c  for description look at main header description
c
c     -----------------------------------------------------------------------
      DATA DLT/0.05d0/  ! Parameter for theta-integration (Gauss-Legendre rule)
      DATA NPOI/125/    ! Total number of shapefunction points
      DATA NRAD/10/     ! Muffintinization points
      DATA NMIN/3/      ! Minimum number of points in panel
      DATA TOLVDIST/1.D-5/ ! Max. tolerance for distance of two vertices
      DATA TOLAREA/1.D-10/  ! Max. tolerance for area of polygon face
      DATA TOLEULER/1.D-10/ ! Used in calculation of Euler angles, subr. EULER
c     -----------------------------------------------------------------------       
C=======================================================================
C
C INITIALIZATION, begin
C
C=======================================================================
C
       PI = 4.0D0*DATAN(1.0D0)
       FPI = 4.0D0*PI
       RFPI = DSQRT(FPI)
       QBOUND = 1.D-9
       SIZEFAC(0) = 1.d0    ! for atoms outside slab
c
c     Read Lattice form inputcard
c
       WRITE(6,2000)
       WRITE(6,2001)
       WRITE(6,2002)
       WRITE(6,2000)
       OPEN(15,FILE='shapefun',STATUS='UNKNOWN')
c
       CALL READINPUT(RBASIS,ABASIS,BBASIS,CBASIS,
     &                ALATC,CLS,NCLS,REFPOT,
     &                IRNS,NTCELL,NAEZ,KAOEZ,IRM,Z,
     &                NINEQ,NTCELLR,
     &                IFILE,INS,
     &                KFG,KWS,KXC,LMAX,LPOT,
     &                NATYP,NSPIN,
     &                TXC,LMXC,
     &                KMT,MTFAC,I13,
     &                RCUTZ,RCUTXY,RMTCORE,
     &                SIZEFAC)
c
c
       CALL LATTIX2000(ALATC,BRAVAIS,RR,NR,VOLUC)
c
       CALL SCALEVEC2000(RBASIS,ABASIS,BBASIS,CBASIS,
     &                   NAEZ,BRAVAIS,KAOEZ)
c
c
c
       CALL CLSGEN2000(NAEZ,RR,NR,RBASIS,
     &                 Z,CLS,NCLS,
     &                 NACLS,ATOM,
     &                 RCLS,RCUTZ,RCUTXY,
     &                 ALATC)
c
C
       DO II1=1,NTOTD
          DX(II1) = 0.D0      ! ARRAY
          DY(II1) = 0.D0      ! ARRAY
          DZ(II1) = 0.D0      ! ARRAY
       ENDDO
C
       DO I=1,NAEZ
         REPATOM(I) = I
       ENDDO
       NREPATOM = NAEZ
C
C
C=======================================================================
C
C INITIALIZATION, end
C
C=======================================================================


C=======================================================================
C
C GEOMETRY, begin
C
C=======================================================================
       WRITE (6,*)
       WRITE(6,'(79(1H=),/,15X,A)')
     &     'maindriver: GEOMETRY, begin'
       WRITE (6,'(79(1H=),/)')
       WRITE (6,*)
c
c
       DO II1=1,NATYPD 
         LSHIFT(II1) = .FALSE.       ! ARRAY
       ENDDO
       NSHAPE = 0
c
c
c
C=======================================================================
C=======================================================================
       DO 50 IAT = 1, NAEZ
C=======================================================================
C=======================================================================
c
         MAKESHAPE =.true.
         ICLUSTER = CLS(IAT)
c
C=======================================================================
         DO I1 = 1,NSHAPE       ! number of shape fun made so far!
C=======================================================================
c
           IC0 = SHAPECL(I1)   ! cluster index for shape
c
           DO I=1,3
             SHIFT(I) = SHAPESHIFT(I,I1) ! extra shift for this atom
           ENDDO

           DIFF = SQRT((SHIFT(1) - DX(REPATOM(IAT)))**2 +
     &                 (SHIFT(2) - DY(REPATOM(IAT)))**2 +
     &                 (SHIFT(3) - DZ(REPATOM(IAT)))**2)
c
           IF ((ICLUSTER.EQ.IC0).AND.(DIFF.LT.1.D-6)) THEN 
c
c Now test if size of the atoms is the same
c test ic0 cluster for IAT atom if it matches any previous
c shape
c
             NVEC = NACLS(ICLUSTER)
             RTEST = 0.D0
             DO I=1,NVEC
               II = ATOM(I,IAT)
               IF (II.LT.0) II = 0
               RTEST = RTEST + ABS(SHAPEWEIGHT(I,I1) - SIZEFAC(II))
             ENDDO
             IF (RTEST.LT.1.D-4) THEN
               MAKESHAPE=.FALSE.
               IDSHAPE(IAT) = I1
             ENDIF
c
           ENDIF
c
C=======================================================================
         ENDDO                 ! NSHAPE loop
C=======================================================================
c
         DIFF = SQRT(DX(REPATOM(IAT))**2 +
     &               DY(REPATOM(IAT))**2 +
     &               DZ(REPATOM(IAT))**2)
c
         IF (DIFF.GT.1.D-6) LSHIFT(IAT) =.TRUE.
c
c
         IF (.NOT.(MAKESHAPE)) GOTO 50      ! goto end of IAT loop 
c
c
         NSHAPE = NSHAPE + 1
         WRITE(6,900)
         WRITE(6,901)NSHAPE,ICLUSTER,DX(REPATOM(IAT)),
     &        DY(REPATOM(IAT)),DZ(REPATOM(IAT)),IAT
c
         SHAPESHIFT(1,NSHAPE) = DX(REPATOM(IAT))
         SHAPESHIFT(2,NSHAPE) = DY(REPATOM(IAT))
         SHAPESHIFT(3,NSHAPE) = DZ(REPATOM(IAT))
         SHAPECL(NSHAPE) = ICLUSTER
c
c
         IDSHAPE(IAT) = NSHAPE
c
c
         NVEC = NACLS(ICLUSTer)
         IF (NVEC.GT.NFACED) THEN
            WRITE(6,*) 'Increase NFACED ',NVEC,NFACED
            STOP
         END IF
c
c     First vector is always zero, take it out
c
         DO I=2,NVEC
            DO J=1,3
               RVEC(J,I-1) = RCLS(J,I,ICLUSTer)
            END DO
         END DO
c

         DO I=1,NVEC 
            II = ATOM(I,IAT)
            IF (II.LT.0) II = 0 
            SHAPEWEIGHT(I,NSHAPE) = SIZEFAC(II)
         END DO
c
         NVEC = NVEC - 1
c
         WEIGHT0 = SIZEFAC(IAT)
c
         DO I=1,NVEC 
            II = ATOM(I+1,IAT) 
            IF (II.LT.0.OR.II.GT.NAEZ) II = 0  
c
c     in case of slab or decimation you have to take care of the boundary
c     atoms give all atoms outside the system weight 1.0
c
            WEIGHT(I) = SIZEFAC(II)
         ENDDO
         
c
         WRITE(6,*) 'Entering VORONOI08 for atom=',IAT
c
         CALL VORONOI08(
     >                  NVEC,RVEC,NVERTD,NFACED,
     >                  WEIGHT0,WEIGHT,TOLVDIST,TOLAREA,
     <                  RMT0,ROUT,VOLUME,NFACE,
     <                  A3,B3,C3,D3,NEDGE,XEDGE,YEDGE,ZEDGE)
c
c
c
c     ------------------------------------------------------------------- 
c
c
         RMTCL(NSHAPE) = RMT0*ALATC ! aa 
         VOLUMECL(NSHAPE) = VOLUME
         RWSCL(NSHAPE) =  ALATC*(3.d0*VOLUME/FPI)**(1.D0/3.D0)
c
c     Save information for drawing
c
c
         NFACECL(IDSHAPE(IAT)) = NFACE
c
         DO I=1,NFACE
           A3CL(I,IDSHAPE(IAT)) = A3(I)
           B3CL(I,IDSHAPE(IAT)) = B3(I)
           C3CL(I,IDSHAPE(IAT)) = C3(I)
           D3CL(I,IDSHAPE(IAT)) = D3(I)
           NEDGECL(I,IDSHAPE(IAT)) = NEDGE(I)
C
           DO J=1,NEDGE(I)
             XEDGECL(J,I,IDSHAPE(IAT)) = XEDGE(J,I)
             YEDGECL(J,I,IDSHAPE(IAT)) = YEDGE(J,I)
             ZEDGECL(J,I,IDSHAPE(IAT)) = ZEDGE(J,I)
           ENDDO
C
         ENDDO
C
C
         IF (OPT('noshape ')) GOTO 50 ! Only voronoi analysis, no shapes.
c
c 9000   FORMAT(4F20.13) 
c
c
C     The equation        A1*x + A2*y + A3*z = A4
c     is transformed to   A1*x'+ A2*y'+ A3*z'= A4 -(A1*dx+A2*dy+A3dz)
c
c
         IF (LSHIFT(IAT)) THEN
           DO I=1,NFACE
             D3(I) = D3(I) - A3(I)*DX(REPATOM(IAT))  
     &             - B3(i)*DY(REPATOM(IAT)) 
     &             - C3(i)*DZ(REPATOM(IAT))
             DO J=1,NEDGE(I)
               XEDGE(J,I) = XEDGE(J,I) - DX(REPATOM(IAT))
               YEDGE(J,I) = YEDGE(J,I) - DY(REPATOM(IAT))
               ZEDGE(J,I) = ZEDGE(J,I) - DZ(REPATOM(IAT)) 
             ENDDO
           ENDDO
         ENDIF
c
C=======================================================================
         IF (INS.GT.0) THEN
C=======================================================================
c
c     Calculate shape functions for each Voronoi shape.
c
           LMAXSHAPE = 4*LMAX
           KEYPAN = 0
           DO II1=1,NPAND
             NM(II1) = 0
           ENDDO
c
c
           CALL SHAPE(NPOI,A3,B3,C3,D3,
     &                TOLVDIST,TOLEULER,NMIN,
     &                NEDGE,XEDGE,YEDGE,ZEDGE,NFACE,LMAXSHAPE,
     &                DLT,KEYPAN,NM,ICLUSTER,NCELL_S,SCALE_S,
     +                NPAN_S,MESHN_S(NSHAPE),
     &                NM_S,XRN_S(1,NSHAPE),DRN_S(1,NSHAPE),NFUN_S,
     &                LMIFUN_S,THETAS_S)
c
c
c The first mesh point of the shape is the mt-radious, save it!
c
c
           SCALEALL(NSHAPE) = SCALE_S
           WRITE(6,*) SCALEALL(NSHAPE),NSHAPE
c
c
           RMTCL(NSHAPE) = XRN_S(1,NSHAPE)*ALATC  
           MTRADIUS = RMTCORE(IAT) ! scale with alat 
           WRITE(6,1020) NFACE
           WRITE(6,1000) VOLUME
           WRITE(6,1010) RWSCL(NSHAPE)       
           WRITE(6,1030) RMTCL(NSHAPE)

           IF (MTRADIUS.GE.RMTCL(NSHAPE)) THEN
             write(6,*) 'RMT of atom',IAT,' reduced to ',
     &              0.99d0*RMTCL(NSHAPE)
             MTRADIUS = 0.99d0*RMTCL(NSHAPE)
           ENDIF
c
c
c
           IF ((MTRADIUS.LT.RMTCL(NSHAPE)).AND.(MTRADIUS.NE.0.0)
     &         .AND.(INS.GT.0)) THEN
c
             WRITE(6,1050)
             WRITE(6,1070) rmtcore(IAT)
             WRITE(6,1080) 
     &              (RMTCL(NSHAPE)-rmtcore(IAT))/RMTCL(NSHAPE)*100.0D0
c
c     Now change MT and write out the shape function
c
             CALL MTMESH(NRAD,npan_s,meshn_s(NSHAPE),nm_S,
     &                   xrn_S(1,NSHAPE),
     &                   drn_S(1,NSHAPE),nfun_s,thetas_s,LMIFUN_S,
     &                   mtradius/alatc)               
c
c     redefine
             RMTCL(NSHAPE) = XRN_S(1,NSHAPE)*ALATC
             RWSCL(NSHAPE) = XRN_S(NPOI+NRAD,NSHAPE)*ALATC
             WRITE(6,*) 'rmt  = ',RMTCL(NSHAPE)
             WRITE(6,*) 'rmax = ',RWSCL(NSHAPE),
     &       XRN_S(NPOI+NRAD,NSHAPE),ALATC
           ELSE  
             WRITE(6,1090)
             WRITE(6,1070) RMTCORE(IAT)
           END IF
c
c
c
           WRITE(6,*)
           WRITE(6,*)
           WRITE(6,*)
c
c     Now the shape has an extra panel, and starts at mtradius. 
c
c
C=======================================================================
         END IF ! INS.GT.0
C=======================================================================
c
c
         WRITE(6,*) '<<<<<<<<<<<<<<<<<<<<<<--->>>>>>>>>>>>>>>>>>>>>'
c
c
c
C=======================================================================
C=======================================================================
 50    CONTINUE               ! Do loop in atoms
C=======================================================================
C=======================================================================
c
       CLOSE(25)
c
c     Print general information about lattice
c
       WRITE(6,*) '*******************************************'
       WRITE(6,*) ' Analyzing lattice finised ...     '
       WRITE(6,*) ' Number of ATOMS..............',NAEZ
       WRITE(6,*) ' Number of CLUSTERS ..........',NCLS
       WRITE(6,*) ' Number of SHAPES.. ..........',NSHAPE
c
       DO IAT=1,NAEZ
         WRITE(6,901) IDSHAPE(IAT),CLS(IAT),DX(REPATOM(IAT)),
     &          DY(REPATOM(IAT)),DZ(REPATOM(IAT)),REPATOM(IAT)
         IRWS(IAT)= IRM
       ENDDO
c
       WRITE(6,*) ' ****** Volume for each atom ******' 
       IF (INS.GT.0) THEN
         WRITE(6,*) 'RMAX = maximum radius of the shape, RMT is updated'
         WRITE(6,*) 'RMAX is the Atomic Sphere radious'
       ENDIF
       VTOT = 0.d0
c
       DO IAT = 1,NAEZ
         WRITE(6,1150) IAT,VOLUMECL(IDSHAPE(IAT)),
     &     RMTCL(IDSHAPE(IAT)),RWSCL(IDSHAPE(IAT))
         VTOT = VTOT + VOLUMECL(IDSHAPE(IAT))
       ENDDO
c
       WRITE(6,1160) VTOT,VTOT*ALATC*ALATC*ALATC
       WRITE(6,1170) VOLUC
       WRITE(6,*)
       WRITE(6,*) '+++++++++++ Note about volumes +++++++++++++++++'    
       WRITE(6,*) 
     &      ' Volume is probably different if you are in 2d mode '
       WRITE(6,*) ' In case of 3d mode and volume inconsistency '
       WRITE(6,*) ' try increasing the cluster atoms by changing '
       WRITE(6,*) ' RCLUSTZ and RCLUSTXY  in the inputcard'
       WRITE(6,*) '++++++++++++++++++++++++++++++++++++++++'
c
c     ASA Radious 
c
c         write(6,*) '**** All ASA spheres are set the same **** '
c         vtot = vtot/natoms
c         do iat=1,nshape
c            RWSCL(IAT)= ALATC*(3.d0*Vtot/FPI)**(1.D0/3.D0)       
c         end do  
c         write(6,1112) rwscl(1)
c
c     Comment this out for variing ASA radious!! 
c
c
C
C write shapehead
C
       OPEN (26,STATUS='UNKNOWN',FILE='shapehead')
       WRITE(26,FMT=9000) NSHAPE
       WRITE(26,FMT=9010) (SCALEALL(J),J=1,NSHAPE)
       CLOSE(26)
C
C
       OPEN(27,FILE='atominfo.new',FORM='formatted')
       DO IAT=1,NAEZ
         WRITE(27,2700) Z(IAT),LMXC(IAT),(KFG(J,IAT),J=1,4),CLS(IAT),
     +                  CLS(IAT),IDSHAPE(IAT),MTFAC(IAT),IRNS(IAT),
     +                  RMTCORE(IAT),SIZEFAC(IAT)
       ENDDO
       CLOSE(27)
C
C
       WRITE (6,*)
       WRITE(6,'(79(1H=),/,15X,A)')
     &     'maindriver: GEOMETRY, end'
       WRITE (6,'(79(1H=),/)')
       WRITE (6,*)
C
C=======================================================================
C
C GEOMETRY, end
C
C=======================================================================
C
C
C
C
C=======================================================================
C
C POTENTIAL, begin
C
C=======================================================================
C
      CALL POTENTIAL(
     >               NAEZ,NSPIN,INS,LPOT,Z,ALATC,KFG,KXC,TXC,
     >               QBOUND,I13,
     >               IRWS,IRNS,RWSCL,RMTCL,RMTCORE,
     >               IDSHAPE,VOLUMECL,
     >               MESHN_S,XRN_S,DRN_S)
C
C=======================================================================
C
C POTENTIAL, end
C
C=======================================================================
C
C
C
C
C=======================================================================
C
C VISUALIZATION, begin
C
C=======================================================================
C
       VORO = .TRUE.
       IF (VORO) THEN
         CALL VISUAL(
     >               NAEZ,Z,RBASIS,
     >               IDSHAPE,NFACECL,
     >               NEDGECL,XEDGECL,YEDGECL,ZEDGECL)
       ENDIF
C
C=======================================================================
C
C VISUALIZATION, end
C
C=======================================================================
C
C
  900    format( ' ****** NEW POLYHEDRON AND SHAPE FUNCTION ******'//)
  901    format('Shape...',I4,' has cluster..',I4,' extra shift..',3F8.4
     &          ,' for atom..',I4)

 1000    format('Volume ....(alat^3).......',F14.8)
 1010    format('ASA Sphere (a.u.).........',F14.8)
 1030    format('MT  Sphere (a.u.).........',F14.8)
 1020    format('Faces for polyhedron......',I8)
 1050    format(/'Your shape function will be updated '/
     &        'to match to the smaller Muffin Tin sphere.')
 1070    format('New RMT (a.u.)............',F14.8)
 1080    format('Percentage Change ........',F8.2)
 1090    format('The Shape Function was not updated ' /
     &        'Your Muffin-Tin Sphere was too big or zero' /
     &        'or you are doing ASA calculation' )
 1112    format(' ASA Radious set to ........',F12.8/
     &        '***************************************'/)
 1150    format
     & (' Atom ..',I5,' Volume(alat^3)  : ',F12.8,
     &        '   RMT : ',F8.5,'   RMAX : ',F8.5) 
 1160    format(/'Total volume (alat^3)    ...',F16.8/
     &          'Total Volume (a.u.^3)   ....',F16.8)
 1170    format(/'Bravais cross prod.(alat^3):',F16.8)
 2000    format
     &('**********************************************************')
 2001    format
     &('*         SCREENED KKR POTENTIAL PREPARATION AND         *',/ 
     & '*             LATTICE VISUALIZATION UTILITY              *')
 2002    format
     &('*                                        v.2.0  Mar 2002 *')
 2700    FORMAT ((f4.1),2x,i2,3x,4(i2),3(3x,i5),3x,f4.2,2x,
     &           i4,2x,f5.3,2x,f3.1)
 9000    FORMAT (16i5)
 9010    FORMAT (4d20.12)
C
         END 

